import javax.swing.*;
import java.awt.*;
import java.util.*;
import java.text.*;

/**
 * This simple class draws a simple line graph of one or more
 *  series of data.  The graph autoscales the Y axis maximum, and
 *  can also be set to autoscale the minimum axis.
 */
public class JGraph extends JPanel {
    private Vector  series[];
    private String  seriesNames[];
    private double  maxY=0,minY=0;
    private int     maxX=0;
    private String  labelX=null,labelY=null;
    private boolean autoMinY=false;

    private Color  colours[]= {Color.black,Color.red,Color.green,
                               Color.blue,Color.lightGray,Color.pink,
                               Color.cyan,Color.orange,Color.yellow,
                               Color.darkGray,Color.magenta,Color.gray};

    /**
     * Creates a new blank graph with
     *  a number of series.
     *@param nSeries number of series on the graph
     */
    public JGraph(int nSeries) {
        int t;

        series=new Vector[nSeries];
        seriesNames=new String[nSeries];

        for(t=0;t<nSeries;t++) {
            series[t]=new Vector();
            seriesNames[t]=null;
        }

        setBackground(Color.white);
    }

    /**
     * Creates a <CODE>JGraph</CODE> with labels on
     *  the X and Y axes.
     *@param nSeries the number of series on the graph
     *@param xLabel  the label for X axis
     *@param yLabel  the label for Y axis
     */
    public JGraph(int nSeries,String xLabel,String yLabel) {
        this(nSeries);

        labelX=xLabel;
        labelY=yLabel;
    }


    /**
     * Enables or disables whether the minimum value on the
     *  Y axis should be automatically adjusted to the minumum
     *  value in the graph, or whether it should be held at 0.
     *@param v <CODE>true</CODE> to enable automatic adjustment
     *          of the minimum value.
     */
    public void setAutoScaleMinY(boolean v) {
        autoMinY=v;
    }

    /**
     * Clears all currently logged data.
     */
    public void clearData() {
        int t;

        for(t=0;t<series.length;t++)
            series[t]=new Vector();

        maxY=0;
        maxX=0;
        minY=0;
    }


    /**
     * Assigns a name to a series on the graph.
     *@param name the name of the series
     *@param s    the number of the series
     */
    public void nameSeries(String name,int s) {
        seriesNames[s]=name;
    }


    /**
     * Add an item of data for a given series.
     *@param value the value of the item
     *@param s     the series that the data is for.
     */
    public void addData(double value,int s) {
        series[s].add((Object)new Double(value));

        /* Keep tabs on the max & min value seen so far */
        if(value>maxY)
            maxY=value;
        if(autoMinY && (value<minY || series[s].size()==1))
            minY=value;
        if(series[s].size()-1>maxX)
            maxX=series[s].size()-1;
    }


    /**
     * Obtain a string representation of the graph.
     *@return A string representation of the graph.
     */
    public String toString() {
        String s=new String();
        int    t,u;

        for(t=0;t<10;t++) {
            s=s+ t+ ":  ";
            for(u=0;u<series.length;u++) {
                s=s+series[u].elementAt(t)+"\t";
            }
            s=s+"\n";
        }

        return s;

    }

    public void paint(Graphics g) {

        super.paint(g);

        int x1,x2,y1,y2,m;
        int t;

        m=20;
        x1=0;
        y1=0;
        x2=x1+getWidth();
        y2=y1+getHeight();

        /* Draw x and y axis */
        g.drawLine(m,y2-m+1,m,m);
        g.drawLine(m-1,y2-m,x2-m,y2-m);
        g.drawLine(m-1,m,m,m);
        g.drawLine(x2-m,y2-m,x2-m,y2-m+1);

        FontMetrics  fm=g.getFontMetrics();
        int          fh=fm.getHeight();
        NumberFormat form=new DecimalFormat("#######.##");

        String       sn=form.format(maxY);

        g.drawString(sn,Math.max(m-fm.stringWidth(sn),0),m-2);
        g.drawString(Integer.toString(maxX),x2-m,y2-m+fh);

        /* Draw zero if appropriate */
        if(minY==0 || !autoMinY)
            g.drawString("0",m-fm.charWidth('0'),y2-m+fh);
        else {
            sn=form.format(minY);
            g.drawString(sn,Math.max(m/2-fm.stringWidth(sn)/2,0),y2-m+(int)(fh/3));
            g.drawString("0",m-fm.charWidth('0')/2,y2-m+fh);
        }

        if(labelX!=null)
            g.drawString(labelX,x2/2-fm.stringWidth(labelX)/2,y2-m+fh);

        if(labelY!=null) {
            int top=y2/2-(fh*labelY.length())/2;

            for(t=0;t<labelY.length();t++) {
                String letter=labelY.substring(t,t+1);
                g.drawString(letter,m/2-fm.stringWidth(letter)/2,top+t*fh);

            }

        }

        /* Plot the graphs */
        for(t=0;t<series.length;t++) {
            int u=0;
            g.setColor(colours[t%colours.length]);

            while(u+1<series[t].size()) {
                g.drawLine(m+(int)(((x2-2*m)/(float)maxX)*u),
                           y2-m-(int) ( ((y2-2*m)/(maxY-minY)) * (((Double)series[t].elementAt(u)).doubleValue()-minY)),
                           m+(int)(((x2-2*m)/(float)maxX)*(u+1)),
                           y2-m-(int) ( ((y2-2*m)/(maxY-minY)) * (((Double)series[t].elementAt(u+1)).doubleValue()-minY)));
                u++;
            }
        }

        /* Now draw the key */
        int yPosition=y2-m*2;

        for(t=0;t<series.length;t++) {
            if(seriesNames[t]!=null) {
                g.setColor(colours[t%colours.length]);
                g.drawString(seriesNames[t],x2-m*2-fm.stringWidth(seriesNames[t]),yPosition);
                yPosition-=fh;
            }
        }

    }

    /**
     * Simply creates a window with a fictional <CODE>JGraph</CODE>
     *  in it to test the graph works correctly.
     */
    public static void main(String args[]) {
        JFrame jf=new JFrame("JGraph Demo");
        JGraph jg=new JGraph(2,"Time","Power");
        int t;

        //jg.setAutoScaleMinY(true);

        for(t=0;t<200;t++) {
          jg.addData(Math.sin((t+200)/160.0),0);
          jg.addData(Math.sin(t/150.0)+1,1);
        }

        jg.nameSeries("Petrol",0);
        jg.nameSeries("Diesel",1);

        jf.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        jf.getContentPane().add(jg);
        jf.setSize(200,200);
        jf.show();

    }


}
